////////////////////////////////////////////////////////////////////////////////
//
// history:
//
// 2020-03-14  uj  1.0.0  setup
//
////////////////////////////////////////////////////////////////////////////////

/**
 * (c) Janicke Consulting, Germany 2022
 * 
 * Class that takes care for the mode definitions of the LTO either the 
 * certification LTO or the more detailed LASPORT LTO.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 3 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

package de.janicke.airport.eedb;

public class MOD {
  
  public static final String VERSION = "1.0.0";
  public static enum MODE { TO, CO, AP, ID };
  public static enum MODX { AF, AG, ID, TG, CI, CF };
  public static final String LTO = "LTO";
  
  public static final int[] I = { MODE.TO.ordinal(), MODE.CO.ordinal(), MODE.AP.ordinal(), MODE.ID.ordinal() };
  public static final double[] ICAOTHR = { 1.0, 0.85, 0.30, 0.07 };
  public static final double[] ICAOTIM = { 42.0, 132.0, 240.0, 1560.0 };
  
  public static final double[] MACH = { 0.1, 0.2, 0.1, 0. };
  
  
  /**
   * Maps the extended LTO to  certification LTO mode
   * @param m the extended (LASPORT) mode
   * @return the certification LTO mode
   */
  public static MODE LSP2LTO(MODX m) {
    if (m == MODX.AF) 
      return MODE.AP;
    else if (m == MODX.AG || m == MODX.ID)
      return MODE.ID;
    else if (m == MODX.TG)
      return MODE.TO;
    else
      return MODE.CO;
  }
  
}
