
////////////////////////////////////////////////////////////////////////////////
//
// history:
//
// 2022-01-04  uj  0.1.0  first setup
// 2022-04-14  uj  1.0.0  first release
//
////////////////////////////////////////////////////////////////////////////////

/**
 * (c) Janicke Consulting, Germany 2022
 * mail info@janicke.de
 * 
 * Graphical User Interface for the Simple Engine emission CalculaTOR.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 3 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 * 
 * Version 1.0.0 was developed in the project RAPTOR which has
 * received funding from the Clean Sky 2 Joint Undertaking under
 * the European Union's Horizon 2020 research program under grant
 * agreement No 863969.
 *
 */

package de.janicke.airport.sector;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.net.URI;
import java.net.URL;
import java.util.ArrayList;
import java.util.Locale;
import java.util.logging.Level;
import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTabbedPane;
import javax.swing.JTextField;
import javax.swing.JTextPane;
import javax.swing.filechooser.FileFilter;


public class SectorGUI extends JPanel implements KeyListener {
  
  public static final String VERSION = "1.0.0";

  private JTextField tfIn;
  private JTextPane[] tpAll;
  private JScrollPane[] spAll;
  private JButton btnIn, btnLoad, btnRun;
  private JPanel pTop;
  private JTabbedPane tbPane;
  private JFileChooser fChooser;
  private JFrame superFrame;
  private boolean loaded;
  private String sInorg, sIdorg;
  private float FontSize;
  
  public SectorGUI(JFrame f) {
    superFrame = f;
  }
  
  @Override
  public void keyTyped(KeyEvent e) {}

  @Override
  public void keyPressed(KeyEvent e) {}
  
  @Override
  public void keyReleased(KeyEvent e) {
    if (e.isControlDown() && 
        (e.getKeyCode() == KeyEvent.VK_PLUS || 
         e.getKeyCode() == KeyEvent.VK_MINUS)) {
      FontSize += (e.getKeyCode() == KeyEvent.VK_PLUS) ? 1. : -1.;
      for (JTextPane tp : tpAll) {
        tp.setFont(tp.getFont().deriveFont(FontSize));
        tp.repaint();
      }
    }
  }
  
  private static String my_home() {
    Class<SectorGUI> cls = SectorGUI.class;
    String nm = cls.getName();
    URL u2 = cls.getResource(cls.getSimpleName() + ".class"); 
    File root = null;
    try {
      URI uri = u2.toURI();
      String sc = uri.getScheme();
      String s2 = uri.getSchemeSpecificPart();
      if (sc.equals("jar")) { 
        if (s2.startsWith("file:")) s2 = s2.substring(5); 
      } 
      else if (!sc.equals("file")) 
        return null;
      int k = s2.indexOf(nm.replace('.', '/'));
      if (k > 0) s2 = s2.substring(0, k-1);
      if (s2.endsWith(".jar!")) { 
        s2 = s2.substring(0, s2.length() - 1);
        root = (new File(s2)).getParentFile();
      }
      else
        root = new File(s2);
      if (root.getName().equals("classes") && 
          root.getParentFile().getName().equals("build"))
        root = root.getParentFile().getParentFile();
    } 
    catch (Exception e) {
      e.printStackTrace(System.out);
    }
    return (root != null) ? root.getAbsolutePath() : null;
  }

  private boolean startUp() {
    ArrayList<Component> vc = new ArrayList<Component>();
    this.removeAll();
    this.setLayout(new BorderLayout());
    if (superFrame != null)
      superFrame.setTitle("SectorGUI "+VERSION+" (c) Janicke Consulting");
    //
    // text fields and buttons
    JTextField tf = new JTextField();
		FontSize = (int)(1.25f * tf.getFont().deriveFont(tf.getFont().getSize()).getSize() + 0.5); 
    btnIn = new JButton("...");
    btnIn.setMargin(new Insets(0,0,0,0));
    btnLoad = new JButton("Load file");
    btnRun = new JButton("Save and run");
    vc.add(btnIn);
    vc.add(btnLoad);
    vc.add(btnRun);
    tfIn = new JTextField(70); 
    tfIn.setFont(new Font("Monospaced", Font.PLAIN, (int)FontSize));
    //
    File fhome = new File(my_home());
    File fin = new File(fhome, "test"+File.separator+"sector-test.txt");
    if (!fin.exists()) {
      fin = new File(fhome.getParentFile(), "test"+File.separator+"sector-test.txt");
    }
    tfIn.setText(fin.getAbsolutePath()); 
    //
    fChooser = new JFileChooser();
    FileFilter filter = new FileFilter() {
      @Override
      public boolean accept(File f) {
        String name = f.getName().toLowerCase();
        return (f.isDirectory() || (name.startsWith("sector-") && name.endsWith(".txt")));
      }
      @Override
      public String getDescription() {
        return "sector-<id>.txt";
      }
    };
    fChooser.setFileFilter(filter);
    fChooser.setDialogTitle("SECTOR input file");
    fChooser.setFileSelectionMode(JFileChooser.FILES_ONLY);  
    ActionListener al = (ActionEvent e) -> {
      if (e.getActionCommand().equals(btnIn.getText())) {
        File f = new File(tfIn.getText());
        if (f != null && f.exists())
          fChooser.setCurrentDirectory(f.getParentFile());
        fChooser.rescanCurrentDirectory();
        if (fChooser.showDialog(superFrame, "Select") != JFileChooser.APPROVE_OPTION)
          return;
        tfIn.setText(fChooser.getSelectedFile().getAbsolutePath());
      }
      else if (e.getActionCommand().equals(btnLoad.getText())) {
        load();
      }
      else if (e.getActionCommand().equals(btnRun.getText())) {
        run();
      }
    };
    btnIn.addActionListener(al);
    btnLoad.addActionListener(al);
    btnRun.addActionListener(al);
    //
    // top panel
    JPanel p;
    JLabel l;
    pTop = new JPanel(new GridBagLayout());
    GridBagConstraints c = new GridBagConstraints();
    c.anchor = GridBagConstraints.WEST;
    c.fill = GridBagConstraints.NONE;
    c.insets = new Insets(5, 5, 5, 5);
    c.gridwidth = 2;
    c.gridy = 0;
    c.gridx = 0;
    l = new JLabel("Input file");
    vc.add(l);
    p = new JPanel();
    p.add(tfIn);   
    p.add(btnIn);  
    pTop.add(l, c);
    c.gridy++;
    pTop.add(p, c);
    p = new JPanel();  
    p.setAlignmentY(JPanel.CENTER_ALIGNMENT);
    p.add(btnLoad);  
    p.add(btnRun);
    c.gridy++;
    pTop.add(p, c);
    //
    // bottom panel   
    tbPane = new JTabbedPane();
    vc.add(tbPane);
    tpAll = new JTextPane[3];
    spAll = new JScrollPane[3];
    for (int i=0; i<tpAll.length; i++) {
      tpAll[i] = new JTextPane();
      tpAll[i].setEditable(i == 0);
      tpAll[i].setFont(tfIn.getFont());
      tpAll[i].setBackground((i == 0) ? Color.white : new Color(255,255,204));
      tpAll[i].addKeyListener(this);
      p = new JPanel(new BorderLayout());
      p.add(tpAll[i]);
      spAll[i] = new JScrollPane(p);
      spAll[i].getVerticalScrollBar().setUnitIncrement(10);
    }
    //
    // tabbed pane and help panels
    tbPane.add("Input", spAll[0]);
    tbPane.add("Output", spAll[1]);
    tbPane.add("Settings", spAll[2]);
    //
    // split panel
    p = new JPanel(new BorderLayout());
    p.add(pTop, BorderLayout.WEST);
    JScrollPane sptop = new JScrollPane(p);
    //
    // settings
    File fs = new File(my_home()+File.separator+"Sector.settings");
    try {
      BufferedReader br = null;
      br = new BufferedReader(new FileReader(fs));
      String line;
      StringBuilder sb = new StringBuilder();
      while ((line = br.readLine()) != null)
        sb.append(line).append("\n");
      br.close();
      tpAll[2].setText(sb.toString());
      tpAll[2].setCaretPosition(0);
    }
    catch (Exception e) {
      log(Level.SEVERE, "can't read settings file!");
    }
    //
    p = new JPanel(new BorderLayout());
    p.add(sptop, BorderLayout.NORTH);
    p.add(tbPane, BorderLayout.CENTER);
    this.add(p);
    for (Component cc : vc)
      cc.setFont(cc.getFont().deriveFont(FontSize));
    setEnabled(true);
    repaint();
    loaded = false;
    return true;
  }
  
  private String getID(String s) {
    if (s == null || !s.endsWith(".txt"))
      return null;
    int i = s.lastIndexOf("sector-");
    if (i < 0)
      return null;
    String ss = s.substring(i+7, s.length()-4);
    return (ss == null || ss.length() < 1) ? null : ss;
  }
  
  private void setLogo(boolean busy) {
    URL u;
    String name = (busy) ? "logo-busy.gif" : "logo.gif";
    try { 
      u = SectorGUI.class.getResource("images/"+name); 
    }
    catch (Exception e) { u = null; }
    if (u != null)
      superFrame.setIconImage(superFrame.getToolkit().getImage(u));
  }
  
  private void log(Level l, String s) {
    if (s == null)
      return;
    if (l == Level.SEVERE)
      s = "*** "+ s;
    try {
      tpAll[1].getDocument().insertString(tpAll[1].getDocument().getLength(), s+"\n", null);
      spAll[1].getVerticalScrollBar().setValue(spAll[1].getVerticalScrollBar().getMaximum());   
      if (l == Level.SEVERE)
        tbPane.setSelectedIndex(1);
    }
    catch (Exception e) {}
  }
  
  private void load() {
    try {
      loaded = false;
      sInorg = null;
      tpAll[0].setText("");
      tpAll[1].setText("");
      File f = new File(tfIn.getText());
      if (!f.exists() || !f.canRead())
        throw new Exception("file "+f.getName()+" not readable!");
      String id = getID(f.getName());
      if (id == null)
        throw new Exception("file name not suited, required: sector-<id>.txt");
      //
      // input  
      BufferedReader br = null;
      br = new BufferedReader(new FileReader(f));
      String line;
      StringBuilder sb = new StringBuilder();
      while ((line = br.readLine()) != null)
        sb.append(line).append("\n");
      br.close();
      tpAll[0].setText(sb.toString());
      tpAll[0].setCaretPosition(0);
      sInorg = sb.toString();
      sIdorg = id;
      //
      // output
      f = new File(f.getParent(), f.getName().replace(".txt", ".log"));
      if (f.exists() && f.canRead()) {
        br = new BufferedReader(new FileReader(f));
        sb.setLength(0);
        while ((line = br.readLine()) != null)
          sb.append(line).append("\n");
        br.close();
        tpAll[1].setText(sb.toString());
        tpAll[1].setCaretPosition(0);
      }
      //
      tbPane.setTitleAt(0, "Input ("+id+")");
      tbPane.setTitleAt(1, "Output ("+id+")");
      loaded = true;
      setEnabled(true);
    }
    catch (Exception e) {
      log(Level.SEVERE, "can't load input file:\n"+e.getMessage());
      e.printStackTrace(System.out);
    }  
  }

  @Override
  public void setEnabled(boolean b) {
    tbPane.setEnabled(b);
    tfIn.setEnabled(b);
    btnIn.setEnabled(b);
    btnLoad.setEnabled(b);
    btnRun.setEnabled(b);
    tpAll[0].setEnabled(b);
    if (btnRun.isEnabled())
      btnRun.setEnabled(loaded);
    setLogo(!b);
  }

  private void run() { 
    try {
      tpAll[1].setText("");
      tbPane.setSelectedIndex(1);
      setEnabled(false);
      File f = new File(tfIn.getText());
      String id = getID(f.getName());
      if (id == null)
        throw new Exception("file name not suited, required: sector-<id>.txt");
      //
      // save
      String s = tpAll[0].getText();
      if ((sInorg != null && !sInorg.trim().equals(s.trim())) ||
          (sIdorg != null && !sIdorg.equals(id))) {
        BufferedWriter bw;
        bw = new BufferedWriter(new FileWriter(f));
        String[] sa = s.split("[\n]");
        for (String ss : sa) {
          bw.write(ss);
          bw.newLine();
        }
        bw.close();
      } 
      if (!f.exists() || !f.canRead())
        throw new Exception("file "+f.getName()+" not readable!");
      //
      // run
      Sector sec = new Sector(f.getParent(), id);
      sec.run();
      load();
    }
    catch (Exception e) {
      log(Level.SEVERE, "can't run input file:\n"+e.getMessage());
      e.printStackTrace(System.out);
    }  
    finally {
      setEnabled(true);
    }
  }

  public static void main(String[] args) {
    Locale.setDefault(Locale.ENGLISH);
    SectorGUI sgui = null;
    try {
      JFrame f = new JFrame();    
      f.addWindowListener(new WindowAdapter() {
        @Override
        public void windowClosing(WindowEvent e) {
            System.exit(0);
          }
      });
      sgui = new SectorGUI(f);
      sgui.startUp();
      f.add(sgui);
      f.setSize(new Dimension(800,700));
      f.setVisible(true);
      f.setLocation(200,100);
      f.validate();
    }
    catch (Exception e) {
      System.out.printf("*** an error occured:\n");
      System.out.printf("%s\n", e.getMessage());
      e.printStackTrace(System.out);
    }
  } 
}
